function [lamda_p, lamda_q, mu_vl, mu_vu, mu_Pg_low, mu_Pg_upp, mu_Qg_low, mu_Qg_upp, mu_Af, mu_At] = ...
			compute_ds_lamda (duals, bind_Af, bind_At, bind_vu, bind_vl,	...
			ng, nl, nb, pvpq, ref, gbus, dAf_dVm, dAf_dVa, dAt_dVm, dAt_dVa, dSbus_dVm, dSbus_dVa);

% compute lamda of load flow eqations, mu of line constraints and voltage constraints, etc.
% The algorithm is based on dense formulation of opf.

%   MATPOWER Version 1.0.1
%   by Deqiang (David) Gan, PSERC Cornell    9/19/97
%   Copyright (c) 1997 by Power System Engineering Research Center (PSERC)
%   See http://www.pserc.cornell.edu/ for more info.

nb1 = nb-1;

% get mu of functional constraints
mu_Af = zeros(nl, 1); 
mu_At = zeros(nl, 1); 
mu_vu = zeros(nb, 1); 
mu_vl = zeros(nb, 1); 
mu_Af(bind_Af) = duals(ng+2													:ng+1+length(bind_Af));
mu_At(bind_At) = duals(ng+2+length(bind_Af)									:ng+1+length(bind_Af)+length(bind_At));
mu_vu(bind_vu) = duals(ng+2+length(bind_Af)+length(bind_At)					:ng+1+length(bind_Af)+length(bind_At)+length(bind_vu));
mu_vl(bind_vl) = duals(ng+2+length(bind_Af)+length(bind_At)+length(bind_vu)	:ng+1+length(bind_Af)+length(bind_At)+length(bind_vu)+length(bind_vl));

n_bind_func = sum(length(bind_Af)+length(bind_At)+length(bind_vu)+length(bind_vl));
mu_Pg_low = duals(ng+1+n_bind_func+ng+1			:ng+1+n_bind_func+ng+ng);
mu_Pg_upp = duals(ng+1+n_bind_func+1			:ng+1+n_bind_func+ng);
mu_Qg_low = duals(ng+1+n_bind_func+ng+ng+ng+1	:ng+1+n_bind_func+ng+ng+ng+ng);
mu_Qg_upp = duals(ng+1+n_bind_func+ng+ng+1		:ng+1+n_bind_func+ng+ng+ng);

% get lamda of load flow equations
bmtrx = [	dAf_dVm, 	dAf_dVa(:, pvpq)				% line flow
			dAt_dVm, 	dAt_dVa(:, pvpq)				% line flow
			speye(nb), 	sparse(nb, nb1)					% voltage
			-speye(nb),	sparse(nb, nb1)];				% voltage
workcol=[mu_Af; mu_At; mu_vu; mu_vl];
activeset = find(workcol);
jac1 = [real(dSbus_dVm(ref,:)), real(dSbus_dVa(ref, pvpq))];
workcol2 = duals(ng+1) * (jac1)';
if isempty(activeset) ~= 1, workcol2 = workcol2 + (bmtrx(activeset,:))' * workcol(activeset); end;
jacobian 	=[	real(dSbus_dVm(pvpq, :)),	real(dSbus_dVa(pvpq, pvpq))
	 			imag(dSbus_dVm),			imag(dSbus_dVa(:, pvpq))];	
workcol = -(jacobian')\workcol2;		%% lamdap for load flow equations except reference equation
lamda_p = [	workcol(1:(ref-1))
			duals(ng+1)
			workcol(ref : (nb-1))];		%% lamdap for active load flow equations including reference equation
lamda_q = workcol(nb:2*nb-1);


return;
