function [drop_rate, iruns_max, norm_dx_max, norm_grad_max, norm_slackvar_max, stepsize_P, stepsize_Q, restart_stepsize_P,...
restart_stepsize_Q, how_far_walked_max, violation_ac_max, which_guess] =LPtaskpa(baseMVA);

%LPTASKPA sets up task-dependent parameters (for example, stopping criteria, which LP solver to invoke) for LP-based opf
%solvers.

%   MATPOWER Version 1.0.1
%   by Deqiang (David) Gan, PSERC Cornell    9/19/97
%   Copyright (c) 1997 by Power System Engineering Research Center (PSERC)
%   See http://www.pserc.cornell.edu/ for more info.

drop_rate = 0.8;                        %% drop the stepsize by this rate
iruns_max = 4;                          %% at most how many runs are to be executed
norm_dx_max = 0.5;                      %% maximum change of generation allowed (in MVA)
norm_grad_max = 2.0e-3;                 %% maximum gradient allowed
norm_slackvar_max= 1.0e-8;              %% maximum violation (given by LP subproblem) allowed
stepsize_P = 80/baseMVA;                %% initial stepsize on changes of Pg
stepsize_Q = 50/baseMVA;                %% initial stepsize on changes of Qg
restart_stepsize_P = 14/baseMVA;        %% initial stepsize on changes of Pg for restarting
restart_stepsize_Q = 10/baseMVA;        %% initial stepsize on changes of Qg for restarting
how_far_walked_max = 40*baseMVA;        %% specify how far a search can possibly walk
                                                % if sum of change of generation is larger than the number
                                                % LPOPF concludes that it can not find the optimal solution
violation_ac_max = 1.0e-4;              %% maximum violation (given by ac load flow) allowed
which_guess = 1;                        %% use which initial guess ?
						% 1 -- uniform loading factor
						% 2 -- pre-solved initial solutions
return;

